@@ -467,7 +467,7 @@ Boolean SOEntityCatalog::lookup(const EntityDecl &entity,
       entityEntry = names_[tableIndex].lookup(name,
 					      entity.systemIdPointer() != 0);
     else
-      entityEntry = names_[tableIndex].lookup(entity.name(),
+      entityEntry = names_[tableIndex].lookup(name,
 					      syntax.upperSubstTable(),
 					      entity.systemIdPointer() != 0);
     // match for public id has priority over match for entity in same
@@ -762,6 +762,7 @@ CatalogParser::CatalogParser(const CharsetInfo &charset)
   static const char ucletters[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
   // minimum data other than lcletter, ucletter
   static const char minChars[] = "0123456789-.'()+,/:=?";
+  static const char wwwMinChars[] = { 33, 35, 36, 37, 42, 59, 64, 95, 0 };
   static const char sChars[] = " \n\r\t";
   categoryTable_.setChar(0, nul);
   const char *p;
@@ -777,6 +778,12 @@ CatalogParser::CatalogParser(const CharsetInfo &charset)
     categoryTable_.setChar(charset.execToDesc(*p), s);
   for (p = minChars; *p; p++)
     categoryTable_.setChar(charset.execToDesc(*p), min);
+  for (p = wwwMinChars; *p; p++) {
+    WideChar c;
+    ISet<WideChar> set;
+    if (charset.univToDesc(*p, c, set) > 0 && c <= Char(-1))
+      categoryTable_.setChar(Char(c), min);
+  }
   categoryTable_.setChar(charset.execToDesc('\''), lita);
   categoryTable_.setChar(charset.execToDesc('"'), lit);
   minus_ = charset.execToDesc('-');
