@@ -26,13 +26,12 @@
 #  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 #  SUCH DAMAGE.
 
-# $XFree86: doctools/sgmlformat/sgmlfmt/sgmlfmt.pl,v 1.6 1998/11/14 06:27:17 dawes Exp $
+# $XFree86: doctools/sgmlformat/sgmlfmt/sgmlfmt.pl,v 1.17 2000/07/02 16:33:03 dawes Exp $
 
 # Format an sgml document tagged according to the linuxdoc DTD.
 # by John Fieber <jfieber@freebsd.org> for the FreeBSD documentation
 # project.  
 
-
 require 'newgetopt.pl';
 
 #
@@ -87,9 +86,9 @@ sub exitiferror {
 sub usage {
     print "Usage:\n";
     print "sgmlfmt [-d <doctype>] -f <format> [-i <namea> ...] [-links]\n";
-    print "    [-e encoding] [-hdr file] [-ftr file] [-b] file\n";
+    print "  [-e encoding] [-hdr file] [-ftr file] [-b] [-n] [-l level] file\n";
     print "where <doctype> is one of: linuxdoc (default), docbook.\n";
-    print "and <format> is one of: ascii, html, koi8-r, latin1, ps, roff\n";
+    print "and <format> is one of: ascii, html, koi8-r, latin1, nippon, ps, roff\n";
 }
 
 #
@@ -159,11 +158,16 @@ sub gen_roff {
 # Generate something from roff output
 #
 
+# Extra groff options required on some systems
+
+$groff_opts = "";
+
 sub do_groff {
-    local($driver, $postproc) = @_;
+    local($driver, $postproc, $preamble) = @_;
     @cleanfiles = (@cleanfiles, "${fileroot}.trf", "${fileroot}.tmp", 
     	"${fileroot}.qrf", "${fileroot}.${driver}");
     open (outfile, ">$fileroot.trf");
+    print outfile $preamble;
     &sgmlparse(infile, "roff");
     while (<infile>) {
 	print outfile;
@@ -172,20 +176,52 @@ sub do_groff {
     close(outfile);
     &exitiferror($?);
     
-    system("groff -T ${driver} -t -M ${sgmldir}/tmac ${fileroot}.trf" .
-	   " ${postproc} > ${fileroot}.${driver}");
+    open(infile, "groff ${groff_opts} -T ${driver} -t -M ${sgmldir}/tmac " .
+	   "${fileroot}.trf ${postproc} |");
+    open(outfile, "> ${fileroot}.${driver}");
+    if ("$opt_n") {
+	$save=$/;
+	$/="";
+    }
+    while (<infile>) {
+	print outfile;
+    }
+    close(infile);
+    close(outfile);
     &exitiferror($?);
 
+#    system("groff -T ${driver} -t -M ${sgmldir}/tmac ${fileroot}.trf" .
+#	   " ${postproc} > ${fileroot}.${driver}");
+
     # If foo.tmp has been created, then there are cross references
     # in the file and we need a second pass to resolve them correctly.
 
     if (stat("${fileroot}.tmp")) {
-        system("groff -T ${driver} -t -M ${sgmldir}/tmac ${fileroot}.trf" .
-	       " ${postproc} > ${fileroot}.${driver}");
+	open(infile, "groff ${groff_opts} -T ${driver} -t -M ${sgmldir}/tmac " .
+		"${fileroot}.trf ${postproc} |");
+	open(outfile, "> ${fileroot}.${driver}");
+	while (<infile>) {
+	    print outfile;
+	}
+	close(infile);
+	close(outfile);
+	&exitiferror($?);
+#        system("groff -T ${driver} -t -M ${sgmldir}/tmac ${fileroot}.trf" .
+#	       " ${postproc} > ${fileroot}.${driver}");
     	unlink("${fileroot}.qrf");
     }
     unlink("${fileroot}.trf");
+    if ("$opt_n") {
+	$/=$save;
+    }
     &exitiferror($?);
+
+    if ( $opt_f eq "nippon" ) {
+       system("nkf -e ${fileroot}.${driver} >${fileroot}.euc-jp");
+       unlink("${fileroot}.${driver}");
+    }
+    &exitiferror($?);
+
 }
 
 #
@@ -207,7 +243,7 @@ sub do_groff {
 #
 # Tunable parameters
 #
-$maxlevel = 3;			# max section depth
+$maxlevel = 1;			# max section depth
 $num_depth = 4;			# depth of numbering
 $m_depth = 2;			# depth of menus
 
@@ -216,7 +252,10 @@ $sc = 0;			# section counter
 $filecount = 0;			# file counter
 
 $doctype = "<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 3.2 Final//EN\">";
-$BODY = "<BODY text=\"#000000\" bgcolor=\"#ffffff\">";
+#$BODY = "<BODY text=\"#000000\" bgcolor=\"#ffffff\">";
+# Use the same settings as the rest of the XFree86 pages.  Should add command
+# line options to set these and the other tunable parameters.
+$BODY = "<BODY BGCOLOR=\"#efefef\" TEXT=\"black\" LINK=\"blue\" VLINK=\"#551A8B\" ALINK=\"red\">";
 
 # Other variables:
 #
@@ -340,6 +379,7 @@ sub gen_html {
 sub html2html {
     local($infile, $outfile) = @_;
     local($i);
+    local($inid) = 0;
 
     $sc = 0;
     push(@scs, $sc);
@@ -362,6 +402,17 @@ sub html2html {
 	s/<p><\/p>//;
 
       tagsw: {
+	  # ident
+	  if ($inid == 1 || s/^<\@\@ident>//) {
+	      if (! s/^<\@\@endident>//) {
+	          $ident .= "$_";
+	          $inid = 1;
+	      } else {
+		  $inid = 0;
+	      }
+	      last tagsw;
+	  }
+
 	  # titles and headings
 	  if (s/^<\@\@title>//) {
 	      chop;
@@ -572,7 +623,7 @@ sub html2html {
 
     print tocfile "</HTML>\n";
     open(SECOUT, ">$fileroot.html");
-    print SECOUT "$header[0] $text[0] $footer[0]";
+    print SECOUT "$header[0] $text[0]<HR><PRE>$ident</PRE>\n $footer[0]";
     close(SECOUT);
     close tocfile;
 }
@@ -675,12 +726,26 @@ sub extlink {
     print LINKFILE "ln -fs $fn $file\n";
 }
 
+#
+# Generate misc output
+#
+
+sub gen_misc {
+    local ($format) = @_;
+    &sgmlparse(infile, $format);
+    while (<infile>) {
+	print stdout;
+    }
+    close(infile);
+    &exitiferror($?);
+}
+
 # Now, read the command line and take appropriate action
 
 sub main {
     # Check arguments
     if (!&NGetOpt('d=s', 'f=s', 'links', 'i:s@', 'hdr=s', 'ftr=s', 'e=s',
-		  'b')) {
+		  'b', 'n', 'l=s')) {
 	&usage;
 	exit 1;
     }
@@ -706,12 +771,24 @@ sub main {
     	$decl = "-c " . $sgmldir . "/linuxdoc/catalog";
     }
 
-    if ($opt_b) {
+    if ($opt_b || $opt_f eq "nippon" || $opt_e eq "euc-jp") {
 	$colcmd = "col -bx";
     }
     else {
 	$colcmd = "col";
     }
+    if ($opt_n) {
+	$nroff_preamble = ".nr NOPBRK 1\n";
+    } else {
+	$nroff_preamble = ".nr NOPBRK 0\n";
+    }
+
+    # See if groff has the -U flag.  We need to use it when it is present.
+
+    if (system("groff -U /dev/null > /dev/null 2>&1") == 0) {
+	$groff_opts = "-U";
+    }
+
     # Generate the output
     if ($opt_f eq 'html') {
     	# Set the character encoding
@@ -720,6 +797,9 @@ sub main {
     	}
     	$html_encoding = "<META HTTP-EQUIV=\"Content-Type\" " .
     	    "CONTENT=\"text/html; charset=" . $opt_e . "\">";
+	if (defined($opt_l)) {
+	    $maxlevel = $opt_l;
+	}
 
     	if ($dtd eq "docbook") {
     	    if ($opt_hdr) {$instantopts .= " -D \"inchdr=${opt_hdr}\"";}
@@ -736,25 +816,25 @@ sub main {
 	&gen_roff();
     }
     elsif ($opt_f eq 'ascii') {
-    	&do_groff("ascii", "| $colcmd");
+    	&do_groff("ascii", "| $colcmd", $nroff_preamble);
     }
     elsif ($opt_f eq 'latin1') {
-    	&do_groff("latin1", "| $colcmd");
+    	&do_groff("latin1", "| $colcmd", $nroff_preamble);
     }
     elsif ($opt_f eq 'koi8-r') {
-    	&do_groff("koi8-r", "| $colcmd");
+    	&do_groff("koi8-r", "| $colcmd", $nroff_preamble);
+    }
+    elsif ($opt_f eq 'nippon') {
+    	&do_groff("nippon", "| $colcmd", $nroff_preamble);
     }
     elsif ($opt_f eq 'ps') {
-    	&do_groff("ps", "");
+    	&do_groff("ps", "", "");
     }
-    else {
-	if ($opt_f eq "") {
-	    print "An output format must be specified with the -f
-	    option.\n";
-	}
-	else {
-	    print "\"$opt_f\" is an unknown output format.\n";
-	}
+    elsif ($opt_f ne "") {
+	&gen_misc($opt_f);
+    } else {
+	print "An output format must be specified with the -f
+	option.\n";
 	&usage;
 	exit 1;
     }
